@@ -13,7 +13,6 @@ import grp
 import resource
 import stat
 import pkg_resources
-import select
 import glob
 import platform
 import warnings
@@ -48,6 +47,7 @@ from supervisor.datatypes import profile_options
 from supervisor import loggers
 from supervisor import states
 from supervisor import xmlrpc
+from supervisor import poller
 
 mydir = os.path.abspath(os.path.dirname(__file__))
 version_txt = os.path.join(mydir, 'version.txt')
@@ -100,7 +100,9 @@ class Options:
                        os.path.join(here, 'supervisord.conf'),
                        'supervisord.conf',
                        'etc/supervisord.conf',
-                       '/etc/supervisord.conf']
+                       '/etc/supervisord.conf',
+                       '/etc/supervisor/supervisord.conf',
+                       ]
         self.searchpaths = searchpaths
 
         self.environ_expansions = {}
@@ -290,17 +292,6 @@ class Options:
                     self._set(name, value, 1)
 
         if self.configfile is None:
-            uid = os.getuid()
-            if uid == 0 and "supervisord" in self.progname: # pragma: no cover
-                self.warnings.warn(
-                    'Supervisord is running as root and it is searching '
-                    'for its configuration file in default locations '
-                    '(including its current working directory); you '
-                    'probably want to specify a "-c" argument specifying an '
-                    'absolute path to a configuration file for improved '
-                    'security.'
-                    )
-
             self.configfile = self.default_configfile()
 
         self.process_config()
@@ -403,7 +394,8 @@ class ServerOptions(Options):
     nodaemon = None
     environment = None
     httpservers = ()
-    unlink_socketfiles = True
+    unlink_pidfile = False
+    unlink_socketfiles = False
     mood = states.SupervisorStates.RUNNING
 
     def __init__(self):
@@ -446,8 +438,11 @@ class ServerOptions(Options):
                  "", "profile_options=", profile_options, default=None)
         self.pidhistory = {}
         self.process_group_configs = []
+        self.parse_criticals = []
         self.parse_warnings = []
+        self.parse_infos = []
         self.signal_receiver = SignalReceiver()
+        self.poller = poller.Poller(self)
 
     def version(self, dummy):
         """Print version to stdout and exit(0).
@@ -460,6 +455,18 @@ class ServerOptions(Options):
         return loggers.getLogger(filename, level, fmt, rotating, maxbytes,
                                  backups, stdout)
 
+    def default_configfile(self):
+        if os.getuid() == 0:
+            self.warnings.warn(
+                'Supervisord is running as root and it is searching '
+                'for its configuration file in default locations '
+                '(including its current working directory); you '
+                'probably want to specify a "-c" argument specifying an '
+                'absolute path to a configuration file for improved '
+                'security.'
+                )
+        return Options.default_configfile(self)
+
     def realize(self, *arg, **kw):
         Options.realize(self, *arg, **kw)
         section = self.configroot.supervisord
@@ -529,7 +536,9 @@ class ServerOptions(Options):
     def read_config(self, fp):
         # Clear parse warnings, since we may be re-reading the
         # config a second time after a reload.
+        self.parse_criticals = []
         self.parse_warnings = []
+        self.parse_infos = []
 
         section = self.configroot.supervisord
         if not hasattr(fp, 'read'):
@@ -546,7 +555,9 @@ class ServerOptions(Options):
         except ConfigParser.ParsingError, why:
             raise ValueError(str(why))
 
-        expansions = {'here':self.here}
+        host_node_name = platform.node()
+        expansions = {'here':self.here,
+                      'host_node_name':host_node_name}
         expansions.update(self.environ_expansions)
         if parser.has_section('include'):
             if not parser.has_option('include', 'files'):
@@ -567,7 +578,7 @@ class ServerOptions(Options):
                         'No file matches via include "%s"' % pattern)
                     continue
                 for filename in sorted(filenames):
-                    self.parse_warnings.append(
+                    self.parse_infos.append(
                         'Included extra file "%s" during parsing' % filename)
                     try:
                         parser.read(filename)
@@ -681,10 +692,15 @@ class ServerOptions(Options):
             if not section.startswith('eventlistener:'):
                 continue
             pool_name = section.split(':', 1)[1]
+
             # give listeners a "high" default priority so they are started first
             # and stopped last at mainloop exit
             priority = integer(get(section, 'priority', -1))
+
             buffer_size = integer(get(section, 'buffer_size', 10))
+            if buffer_size < 1:
+                raise ValueError('[%s] section sets invalid buffer_size (%d)' %
+                    (section, buffer_size))
 
             result_handler = get(section, 'result_handler',
                                        'supervisor.dispatchers:default_handler')
@@ -793,7 +809,7 @@ class ServerOptions(Options):
                     socket_owner = (proc_uid, gid_for_uid(proc_uid))
 
             if socket_mode is None:
-                socket_mode = 0700
+                socket_mode = int('700', 8)
 
             return UnixStreamSocketConfig(path, owner=socket_owner,
                                                 mode=socket_mode)
@@ -988,10 +1004,12 @@ class ServerOptions(Options):
         get = parser.saneget
         username = get(section, 'username', None)
         password = get(section, 'password', None)
-        if username is None and password is not None:
-            raise ValueError(
-                'Must specify username if password is specified in [%s]'
-                % section)
+        if username is not None or password is not None:
+            if username is None or password is None:
+                raise ValueError(
+                    'Section [%s] contains incomplete authentication: '
+                    'If a username or a password is specified, both the '
+                    'username and password must be specified' % section)
         return {'username':username, 'password':password}
 
     def server_configs_from_parser(self, parser):
@@ -1040,7 +1058,7 @@ class ServerOptions(Options):
                 except (TypeError, ValueError):
                     raise ValueError('Invalid chmod value %s' % chmod)
             else:
-                chmod = 0700
+                chmod = int('700', 8)
             config['chmod'] = chmod
             config['section'] = section
             configs.append(config)
@@ -1048,6 +1066,11 @@ class ServerOptions(Options):
         return configs
 
     def daemonize(self):
+        self.poller.before_daemonize()
+        self._daemonize()
+        self.poller.after_daemonize()
+
+    def _daemonize(self):
         # To daemonize, we need to become the leader of our own session
         # (process) group.  If we do not, signals sent to our
         # parent process will also be sent to us.   This might be bad because
@@ -1067,7 +1090,7 @@ class ServerOptions(Options):
         # change our umask.
         #
         # This explanation was (gratefully) garnered from
-        # http://www.hawklord.uklinux.net/system/daemons/d3.htm
+        # http://www.cems.uwe.ac.uk/~irjohnso/coursenotes/lrc/system/daemons/d3.htm
 
         pid = os.fork()
         if pid != 0:
@@ -1108,6 +1131,7 @@ class ServerOptions(Options):
         except (IOError, OSError):
             self.logger.critical('could not write pidfile %s' % self.pidfile)
         else:
+            self.unlink_pidfile = True
             self.logger.info('supervisord started with pid %s' % pid)
 
     def cleanup(self):
@@ -1116,7 +1140,9 @@ class ServerOptions(Options):
                 if self.unlink_socketfiles:
                     socketname = config['file']
                     self._try_unlink(socketname)
-        self._try_unlink(self.pidfile)
+        if self.unlink_pidfile:
+            self._try_unlink(self.pidfile)
+        self.poller.close()
 
     def _try_unlink(self, path):
         try:
@@ -1144,7 +1170,7 @@ class ServerOptions(Options):
             # descriptor to be closed, but it will still remain in
             # the socket_map, and eventually its file descriptor
             # will be passed to # select(), which will bomb.  See
-            # also http://www.plope.com/software/collector/253
+            # also https://web.archive.org/web/20160729222427/http://www.plope.com/software/collector/253
             server.close()
 
     def close_logger(self):
@@ -1165,6 +1191,7 @@ class ServerOptions(Options):
     def openhttpservers(self, supervisord):
         try:
             self.httpservers = self.make_http_servers(supervisord)
+            self.unlink_socketfiles = True
         except socket.error, why:
             if why.args[0] == errno.EADDRINUSE:
                 self.usage('Another program is already listening on '
@@ -1179,7 +1206,6 @@ class ServerOptions(Options):
                 else:
                     self.usage('%s errno.%s (%d)' %
                                (help, errorname, why.args[0]))
-            self.unlink_socketfiles = False
         except ValueError, why:
             self.usage(why.args[0])
 
@@ -1221,24 +1247,38 @@ class ServerOptions(Options):
             except OSError:
                 pass
 
-    def select(self, r, w, x, timeout):
-        return select.select(r, w, x, timeout)
-
     def kill(self, pid, signal):
         os.kill(pid, signal)
 
-    def set_uid(self):
-        if self.uid is None:
-            if os.getuid() == 0:
-                return 'Supervisor running as root (no user in config file)'
-            return None
-        msg = self.dropPrivileges(self.uid)
-        if msg is None:
-            return 'Set uid to user %s' % self.uid
-        return msg
+    def waitpid(self):
+        # Need pthread_sigmask here to avoid concurrent sigchld, but Python
+        # doesn't offer in Python < 3.4.  There is still a race condition here;
+        # we can get a sigchld while we're sitting in the waitpid call.
+        # However, AFAICT, if waitpid is interrupted by SIGCHLD, as long as we
+        # call waitpid again (which happens every so often during the normal
+        # course in the mainloop), we'll eventually reap the child that we
+        # tried to reap during the interrupted call. At least on Linux, this
+        # appears to be true, or at least stopping 50 processes at once never
+        # left zombies laying around.
+        try:
+            pid, sts = os.waitpid(-1, os.WNOHANG)
+        except OSError, exc:
+            code = exc.args[0]
+            if code not in (errno.ECHILD, errno.EINTR):
+                self.logger.critical(
+                    'waitpid error %r; '
+                    'a process may not be cleaned up properly' % code
+                    )
+            if code == errno.EINTR:
+                self.logger.blather('EINTR during reap')
+            pid, sts = None, None
+        return pid, sts
 
-    def dropPrivileges(self, user):
-        # Drop root privileges if we have them
+    def drop_privileges(self, user):
+        """Drop privileges to become the specified user, which may be a
+        username or uid.  Called for supervisord startup and when spawning
+        subprocesses.  Returns None on success or a string error message if
+        privileges could not be dropped."""
         if user is None:
             return "No user specified to setuid to!"
 
@@ -1291,29 +1331,29 @@ class ServerOptions(Options):
             return 'Could not set group id of effective user'
         os.setuid(uid)
 
-    def waitpid(self):
-        # Need pthread_sigmask here to avoid concurrent sigchild, but Python
-        # doesn't offer in Python < 3.4.  There is still a race condition here;
-        # we can get a sigchild while we're sitting in the waitpid call.
-        # However, AFAICT, if waitpid is interrupted bu SIGCHILD, as long as we
-        # call waitpid again (which happens every so often during the normal
-        # course in the mainloop), we'll eventually reap the child that we
-        # tried to reap during the interrupted call. At least on Linux, this
-        # appears to be true, or at least stopping 50 processes at once never
-        # left zombies laying around.
-        try:
-            pid, sts = os.waitpid(-1, os.WNOHANG)
-        except OSError, why:
-            err = why.args[0]
-            if err not in (errno.ECHILD, errno.EINTR):
-                self.logger.critical(
-                    'waitpid error; a process may not be cleaned up properly')
-            if err == errno.EINTR:
-                self.logger.blather('EINTR during reap')
-            pid, sts = None, None
-        return pid, sts
+    def set_uid_or_exit(self):
+        """Set the uid of the supervisord process.  Called during supervisord
+        startup only.  No return value.  Exits the process via usage() if
+        privileges could not be dropped."""
+        if self.uid is None:
+            if os.getuid() == 0:
+                self.parse_criticals.append('Supervisor is running as root.  '
+                        'Privileges were not dropped because no user is '
+                        'specified in the config file.  If you intend to run '
+                        'as root, you can set user=root in the config file '
+                        'to avoid this message.')
+        else:
+            msg = self.drop_privileges(self.uid)
+            if msg is None:
+                self.parse_infos.append('Set uid to user %s succeeded' %
+                                        self.uid)
+            else:  # failed to drop privileges
+                self.usage(msg)
 
-    def set_rlimits(self):
+    def set_rlimits_or_exit(self):
+        """Set the rlimits of the supervisord process.  Called during
+        supervisord startup only.  No return value.  Exits the process via
+        usage() if any rlimits could not be set."""
         limits = []
         if hasattr(resource, 'RLIMIT_NOFILE'):
             limits.append(
@@ -1348,10 +1388,7 @@ class ServerOptions(Options):
                 'name':'RLIMIT_NPROC',
                 })
 
-        msgs = []
-
         for limit in limits:
-
             min = limit['min']
             res = limit['resource']
             msg = limit['msg']
@@ -1368,13 +1405,12 @@ class ServerOptions(Options):
 
                 try:
                     resource.setrlimit(res, (min, hard))
-                    msgs.append('Increased %(name)s limit to %(min)s' %
-                                locals())
+                    self.parse_infos.append('Increased %(name)s limit to '
+                                '%(min)s' % locals())
                 except (resource.error, ValueError):
                     self.usage(msg % locals())
-        return msgs
 
-    def make_logger(self, critical_messages, warn_messages, info_messages):
+    def make_logger(self):
         # must be called after realize() and after supervisor does setuid()
         format =  '%(asctime)s %(levelname)s %(message)s\n'
         self.logger = loggers.getLogger(
@@ -1386,11 +1422,11 @@ class ServerOptions(Options):
             backups=self.logfile_backups,
             stdout = self.nodaemon,
             )
-        for msg in critical_messages:
+        for msg in self.parse_criticals:
             self.logger.critical(msg)
-        for msg in warn_messages:
+        for msg in self.parse_warnings:
             self.logger.warn(msg)
-        for msg in info_messages:
+        for msg in self.parse_infos:
             self.logger.info(msg)
 
     def make_http_servers(self, supervisord):
@@ -1424,7 +1460,7 @@ class ServerOptions(Options):
     def mktempfile(self, suffix, prefix, dir):
         # set os._urandomfd as a hack around bad file descriptor bug
         # seen in the wild, see
-        # http://www.plope.com/software/collector/252
+        # https://web.archive.org/web/20160729044005/http://www.plope.com/software/collector/252
         os._urandomfd = None
         fd, filename = tempfile.mkstemp(suffix, prefix, dir)
         os.close(fd)
@@ -1458,7 +1494,7 @@ class ServerOptions(Options):
         elif stat.S_ISDIR(st[stat.ST_MODE]):
             raise NotExecutable("command at %r is a directory" % filename)
 
-        elif not (stat.S_IMODE(st[stat.ST_MODE]) & 0111):
+        elif not (stat.S_IMODE(st[stat.ST_MODE]) & int('111', 8)):
             raise NotExecutable("command at %r is not executable" % filename)
 
         elif not os.access(filename, os.X_OK):
@@ -1873,7 +1909,12 @@ class EventListenerPoolConfig(Config):
         if not isinstance(other, EventListenerPoolConfig):
             return False
 
-        if (self.name == other.name) and (self.priority == other.priority):
+        if ((self.name == other.name) and
+            (self.priority == other.priority) and
+            (self.process_configs == other.process_configs) and
+            (self.buffer_size == other.buffer_size) and
+            (self.pool_events == other.pool_events) and
+            (self.result_handler == other.result_handler)):
             return True
 
         return False
@@ -2109,4 +2150,3 @@ class NoPermission(ProcessException):
     """ Indicates that the file cannot be executed because the supervisor
     process does not possess the appropriate UNIX filesystem permission
     to execute the file. """
-
